#!/usr/bin/python

import Cookie
import Element
from Handlers import *
import HttpConstants
import Request
import RequestParser
import Response
import Session

"""
Handle and direct XML service requests.

Although this executes in a single-threaded manner in CGI, the implementation 
should be threadsafe.
"""
class ServiceDirector:
    def __init__(self):
        pass
        
    """
    Handle a single request.
    """
    def handleRequest(self):
        request = Request.Request()
        sessionId = None
        sessionMap = {}
        newSession = False
        session = Session.Session()
        
        # retrieve or create the session
        if request.sessionId != None:
            sessionId = request.sessionId
            sessionMap = session.getSession(request.sessionId)

        if sessionMap == None:
            # new session or session was erased
            (sessionId, sessionMap) = session.createSession()
            newSession = True

        # create the response        
        response = Response.Response(Response.Response.MODE_XML)
        results = Element.Element("update")
        
        # handle the XML request, get the XML return value
        requestContents = self.parseRequest(request)
        
        # invoke the request handler
        for handler in requestContents.getChildren('handler'):
            results.append(invokeHandler(handler.value, request, response, sessionMap))
            
        # check the results for terminating conditions
        if results.getChildren('end_session'):
            if newSession:
                newSession = False
            else:
                # erase the session cookie
                session.clearSession()
                # subsequent hits with that session ID will become new 
                # sessions instead
        else:
            # store the session
            session.putSession(sessionId, sessionMap)
                
        # new sessions get the session cookie added
        if newSession:
            response.cookies[HttpConstants.SESSION_ID] = sessionId

        # turn the results back into XML and put them in the response
        response.body = self.parseResults(results)

        # close the session DB
        session.closeSessionDb()
        
        return response
        
    """
    Convert the Request object body XML into a hierarchical element tree.
    
    The outer tag (document tag) is discarded.
    """
    def parseRequest(self, request):
        parser = RequestParser.RequestParser()
        if request.body:
            return parser.parse(request.body)
        else:
            # suitable for testing
            return parser.parse('<get><handler>time</handler></get>')
        
    """
    Convert the results map into an XML body.
    """
    def parseResults(self, resultsTree):
        resultsStr = "<" + resultsTree.name
        for attr in resultsTree.attributes.keys():
            resultsStr += " " + attr + "='" + self.escape(resultsTree.attributes[attr]) + "'"
        resultsStr += ">"
        for child in resultsTree.children:
            resultsStr += self.parseResults(child)
        if resultsTree.value:
            resultsStr += str(resultsTree.value)
        resultsStr += "</" + resultsTree.name + ">"
        return resultsStr
        
    """
    Escape apostrophes.
    """
    def escape(self, str):
        return str.replace("'", "\'")

if __name__ == '__main__':
    # handle the request
    director = ServiceDirector()
    
    # return the XML response
    print director.handleRequest()
    
