# Session.py

import hmac
import shelve
import time

"""
Handles session persistence.
"""
sessionDbName = "sessions.dbm"

class Session:
    def __init__(self):
        self.sessionDb = shelve.open(sessionDbName, 'c', 2)
        
    """
    Close the session DB.
    """
    def closeSessionDb(self):
        if self.sessionDb != None:
            self.sessionDb.close()
    
    """
    Get the session by ID if it exists, or None if it doesn't.
    """
    def getSession(self, sessionId):
        if sesionId and self.sessionDb.has_key(sessionId):
            return self.sessionDb[sessionId]
        return None
    
    """
    Persist the session.
    
    The sessionId and session must be set.
    """
    def putSession(self, sessionId, session):
        if sessionId and session:
            self.sessionDb[sessionId] = session
            
    """
    Create a session.
    
    Returns a tuple of session ID and session.
    
    This session has not been persisted, so after it's been modified,
    call putSession().
    """
    def createSession(self):
        sessionId = hmac.new(str(time.time())).hexdigest()
        return (sessionId, {'sessionId': sessionId})
    
    """
    Clear a session.
    
    Removes the session from persistent storage.
    """
    def clearSession(self, sessionId):
        if sessionId:
            del self.sessionDb[sessionId]
